@@ -1,7 +1,7 @@
 from __future__ import absolute_import
 
 from concurrent.futures import Future
-from threading import Thread, current_thread
+from threading import Event, Thread
 
 from tornado import ioloop
 
@@ -28,7 +28,8 @@ class ThreadLoop(object):
 
         @gen.coroutine
         def coroutine(greeting="Goodbye"):
-            raise gen.Result("%s World" % greeting)
+            yield gen.sleep(1)
+            raise gen.Return("%s World" % greeting)
 
         with ThreadLoop() as threadloop:
 
@@ -38,13 +39,14 @@ class ThreadLoop(object):
 
     """
     def __init__(self, io_loop=None):
-        self.main_thread = current_thread()
 
-        self.thread = None
+        self._thread = None
+        self._ready = Event()
+
         if io_loop is None:
-            self.io_loop = ioloop.IOLoop()
+            self._io_loop = ioloop.IOLoop()
         else:
-            self.io_loop = io_loop
+            self._io_loop = io_loop
 
     def __enter__(self):
         self.start()
@@ -55,15 +57,39 @@ class ThreadLoop(object):
 
     def start(self):
         """Start IOLoop in daemonized thread."""
-        assert self.thread is None, 'thread already started'
-        self.thread = Thread(target=self.io_loop.start)
-        self.thread.daemon = True
-        self.thread.start()
+        assert self._thread is None, 'thread already started'
+
+        # configure thread
+        self._thread = Thread(target=self._start_io_loop)
+        self._thread.daemon = True
+
+        # begin thread and block until ready
+        self._thread.start()
+        self._ready.wait()
+
+    def _start_io_loop(self):
+
+        def mark_as_ready():
+            self._ready.set()
+
+        self._io_loop.add_callback(mark_as_ready)
+        self._io_loop.start()
+
+    def is_ready(self):
+        """Is thread & ioloop ready."""
+
+        if not self._thread:
+            return False
+
+        if not self._ready.is_set():
+            return False
+
+        return True
 
     def stop(self):
         """Stop IOLoop & close daemonized thread."""
-        self.io_loop.stop()
-        self.thread.join()
+        self._io_loop.stop()
+        self._thread.join()
 
     def submit(self, fn, *args, **kwargs):
         """Submit Tornado Coroutine to IOLoop in daemonized thread.
@@ -73,8 +99,11 @@ class ThreadLoop(object):
         :param kwargs: Kwargs to pass to coroutine
         :returns concurrent.futures.Future: future result of coroutine
         """
-        if self.thread is None or not self.thread.isAlive():
-            raise ThreadNotStartedError()
+        if not self.is_ready():
+            raise ThreadNotStartedError(
+                "The thread has not been started yet, "
+                "make sure you call start() first"
+            )
 
         future = Future()
 
@@ -94,7 +123,7 @@ class ThreadLoop(object):
             # python3 just needs the exception, exc_info works fine
             future.set_exception(exception)
 
-        self.io_loop.add_callback(
+        self._io_loop.add_callback(
             lambda: fn(*args, **kwargs).add_done_callback(on_done)
         )
 
